﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BitmapTagger
{
	/// <summary>
	/// IDs (keywords) for xmp data
	/// </summary>
	public static class JpegXmpIDs
	{

		/// <summary>
		/// ID for XResolution tag
		/// </summary>
		public const string XResolutionID = "tiff:XResolution";

		/// <summary>
		/// ID for YResolution tag
		/// </summary>
		public const string YResolutionID = "tiff:YResolution";

		/// <summary>
		/// ID for Resolution Unit tag
		/// </summary>
		public const string ResolutionUnitID = "tiff:ResolutionUnit";

		/// <summary>
		/// ID for DateTime tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored in XMP as ModifyDate
		/// </XmpNote>
		public const string DateTimeID = "tiff:DateTime";

		/// <summary>
		/// ID for Modify DateTime
		/// </summary>
		public const string ModifyDateTimeID = "xmp:ModifyDate";

		/// <summary>
		/// ID for Image Description tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored in XMP as Description
		/// </XmpNote>
		public const string ImageDescriptionID = "tiff:ImageDescription";

		/// <summary>
		/// ID for Description tag
		/// </summary>
		public const string DescriptionID = "dc:description";

		/// <summary>
		/// ID for Make tag
		/// </summary>
		public const string MakeID = "tiff:Make";

		/// <summary>
		/// ID for Model tag
		/// </summary>
		public const string ModelID = "tiff:Model";

		/// <summary>
		/// ID for Software tag
		/// </summary>
		/// <XmpNote>
		/// This property is soted in XMP as CreatorTool
		/// </XmpNote>
		public const string SoftwareID = "tiff:Software";

		/// <summary>
		/// ID for Creator Tool tag
		/// </summary>
		public const string CreatorToolID = "xmp:CreatorTool";

		/// <summary>
		/// ID for the Artist tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored as the first item in the dc:creator array
		/// </XmpNote>
		public const string ArtistID = "tiff:Artist";

		/// <summary>
		/// ID for the Creator tag
		/// </summary>
		public const string CreatorID = "dc:creator";

		/// <summary>
		/// ID for Copyright tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored in XMP as dc:rights
		/// </XmpNote>
		public const string CopyrightID = "tiff:Copyright";

		/// <summary>
		/// ID for Rights tag
		/// </summary>
		public const string RightsID = "dc:rights";

		/// <summary>
		/// ID for User Comment tag
		/// </summary>
		public const string UserCommentID = "exif:UserComment";

		/// <summary>
		/// ID for DateTime Original tag
		/// </summary>
		public const string DateTimeOriginalID = "exif:DateTimeOriginal";

		/// <summary>
		/// ID for DateTime Digitized tag
		/// </summary>
		public const string DateTimeDigitizedID = "exif:DateTimeDigitized";

		/// <summary>
		/// ID for Metering Mode tag
		/// </summary>
		public const string MeteringModeID = "exif:MeteringMode";

		/// <summary>
		/// ID for Sensing Method tag
		/// </summary>
		public const string SensingMethodID = "exif:SensingMethod";

		/// <summary>
		/// ID for Exposure Program tag
		/// </summary>
		public const string ExposureProgramID = "exif:ExposureProgram";

		/// <summary>
		/// ID for ISO Speed Ratings tag
		/// </summary>
		public const string ISOSpeedRatingsID = "exif:ISOSpeedRatings";

		/// <summary>
		/// ID for Light Source tag
		/// </summary>
		public const string LightSourceID = "exif:LightSource";

		/// <summary>
		/// ID for Exposure Time tag
		/// </summary>
		public const string ExposureTimeID = "exif:ExposureTime";

		/// <summary>
		/// ID for FNumber tag
		/// </summary>
		public const string FNumberID = "exif:FNumber";

		/// <summary>
		/// ID for Shutter Speed tag
		/// </summary>
		public const string ShutterSpeedID = "exif:ShutterSpeedValue";

		/// <summary>
		/// ID for Brightness tag
		/// </summary>
		public const string BrightnessID = "exif:BrightnessValue";

		/// <summary>
		/// ID for Aperture tag
		/// </summary>
		public const string ApertureID = "exif:ApertureValue";

	}
}
