﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace BitmapTagger
{
	/// <summary>
	/// Contains the tag IDs (names) for the XMPExif tags (not XMP)
	/// </summary>
	public static class XMPExifTagIDs
	{

		/// <summary>
		/// ID for XResolution tag
		/// </summary>
		public static XName XResolutionID = XMPNamespaces.TiffSchema + "XResolution";

		/// <summary>
		/// ID for YResolution tag
		/// </summary>
		public static XName YResolutionID = XMPNamespaces.TiffSchema + "YResolution";

		/// <summary>
		/// ID for Resolution Unit tag
		/// </summary>
		public static XName ResolutionUnitID = XMPNamespaces.TiffSchema + "ResolutionUnit";

		/// <summary>
		/// ID for DateTime tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored in XMP as ModifyDate
		/// </XmpNote>
		public static XName DateTimeID = XMPNamespaces.TiffSchema + "DateTime";



		/// <summary>
		/// ID for Image Description tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored in XMP as Description
		/// </XmpNote>
		public static XName ImageDescriptionID = XMPNamespaces.TiffSchema + "ImageDescription";


		/// <summary>
		/// ID for Make tag
		/// </summary>
		public static XName MakeID = XMPNamespaces.TiffSchema + "Make";

		/// <summary>
		/// ID for Model tag
		/// </summary>
		public static XName ModelID = XMPNamespaces.TiffSchema + "Model";

		/// <summary>
		/// ID for Software tag
		/// </summary>
		/// <XmpNote>
		/// This property is soted in XMP as CreatorTool
		/// </XmpNote>
		public static XName SoftwareID = XMPNamespaces.TiffSchema + "Software";


		/// <summary>
		/// ID for the Artist tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored as the first item in the dc:creator array
		/// </XmpNote>
		public static XName ArtistID = XMPNamespaces.TiffSchema + "Artist";

		/// <summary>
		/// ID for Copyright tag
		/// </summary>
		/// <XmpNote>
		/// This property is stored in XMP as dc:rights
		/// </XmpNote>
		public static XName CopyrightID = XMPNamespaces.TiffSchema + "Copyright";

		/// <summary>
		/// ID for User Comment tag
		/// </summary>
		public static XName UserCommentID = XMPNamespaces.ExifSchema + "UserComment";

		/// <summary>
		/// ID for DateTime Original tag
		/// </summary>
		public static XName DateTimeOriginalID = XMPNamespaces.ExifSchema + "DateTimeOriginal";

		/// <summary>
		/// ID for DateTime Digitized tag
		/// </summary>
		public static XName DateTimeDigitizedID = XMPNamespaces.ExifSchema + "DateTimeDigitized";

		/// <summary>
		/// ID for Metering Mode tag
		/// </summary>

		public static XName MeteringModeID = XMPNamespaces.ExifSchema + "MeteringMode";

		/// <summary>
		/// ID for Sensing Method tag
		/// </summary>
		public static XName SensingMethodID = XMPNamespaces.ExifSchema + "SensingMethod";

		/// <summary>
		/// ID for Exposure Program tag
		/// </summary>
		public static XName ExposureProgramID = XMPNamespaces.ExifSchema + "ExposureProgram";

		/// <summary>
		/// ID for ISO Speed Ratings tag
		/// </summary>
		public static XName ISOSpeedRatingsID = XMPNamespaces.ExifSchema + "ISOSpeedRatings";

		/// <summary>
		/// ID for Light Source tag
		/// </summary>
		public static XName LightSourceID = XMPNamespaces.ExifSchema + "LightSource";

		/// <summary>
		/// ID for Exposure Time tag
		/// </summary>
		public static XName ExposureTimeID = XMPNamespaces.ExifSchema + "ExposureTime";

		/// <summary>
		/// ID for FNumber tag
		/// </summary>
		public static XName FNumberID = XMPNamespaces.ExifSchema + "FNumber";

		/// <summary>
		/// ID for Shutter Speed tag
		/// </summary>
		public static XName ShutterSpeedID = XMPNamespaces.ExifSchema + "ShutterSpeedValue";

		/// <summary>
		/// ID for Brightness tag
		/// </summary>
		public static XName BrightnessID = XMPNamespaces.ExifSchema + "BrightnessValue";

		/// <summary>
		/// ID for Aperture tag
		/// </summary>
		public static XName ApertureID = XMPNamespaces.ExifSchema + "ApertureValue";
	}
}
