﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace BitmapTagger
{
	/// <summary>
	/// Contains the XMP Namespaces for the XMP tags in use
	/// </summary>
	public static class XMPNamespaces
	{
		/// <summary>
		/// Schema namespace for Dublin Core tags
		/// </summary>
		public static XNamespace DublinCoreSchema =  "http://purl.org/dc/elements/1.1/";

		/// <summary>
		/// Schema namespace for basic XMP tags
		/// </summary>
		public static XNamespace XMPBasicSchema = "http://ns.adobe.com/xap/1.0/";

		/// <summary>
		/// Schema namespace for copyright XMP tags
		/// </summary>
		public static XNamespace XMPRightsSchema = "http://ns.adobe.com/xap/1.0/rights/";

		/// <summary>
		/// Schema namespace for tiff derived tags
		/// </summary>
		public static XNamespace TiffSchema = "http://ns.adobe.com/tiff/1.0/";

		/// <summary>
		/// Schema namespace for Exif specific tags
		/// </summary>
		public static XNamespace ExifSchema = "http://ns.adobe.com/exif/1.0/ ";

		/// <summary>
		/// Namespace of (possible) outermost element in XMP that contains one RDF element
		/// </summary>
		/// <remarks>
		/// This is the preferred outermoust element, but is may not always be present
		/// sometimes the RDF element will be the outermost element
		/// </remarks>
		public static XNamespace MetadataNamespace = "adobe:ns:meta/";

		/// <summary>
		/// Namespace of Resource Description Framework (RDF) will be one element that contains 
		/// everything
		/// </summary>
		/// <remarks>
		/// This will either be inside the XMPMetaData element (preffered) or it will be the topmost
		/// element
		/// </remarks>
		/// 
		public static XNamespace RDFNamespace = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

	}
}
