﻿// Original author contact info: Owen Emlen (owene_1998@yahoo.com)
// Note: other individuals may also have contributed to this code
// Project hosted on CodePlex.com as of 1/10/2009 at http://www.codeplex.com/EmlenMud
using System.Runtime.Serialization;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Threading;
using BrainTechLLC.ThreadSafeObjects;
#if USE_HYPER
using Hyper.ComponentModel;
#endif

namespace BrainTechLLC.Logging
{
    /// <summary>
    /// Provides buffered logging to a file
    /// </summary>
    [Serializable]
    [DataContract]
    public class FileLogger : LoggerBase, ILogger
    {        
        private readonly int LogEntriesBeforeWrite = 1;
        private readonly int LogEntriesPerFile = 10000;

        private int _currentLogNumber;
        private string _currentLogFileNameCached;
        private int _entriesWrittenToCurrentLogFile;

        public string FileNameBase { get; set; }

        public static void CreateAndRegisterFileLogger(string path, string fileNameBase)
        {
            if (path.EndsWith("\\"))
                path = path.Substring(0, path.Length - 1);

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            FileLogger logger = new FileLogger(path + "\\" + fileNameBase);
            logger.BeginLogging();
            GlobalLogger.RegisterLogger(logger);
        }

        public FileLogger()
            : base()
        {
        }

        public FileLogger(string fileNameBase)
            : this()
        {
            FileNameBase = fileNameBase;
        }

        public string CurrentLogFileName
        {
            get
            {
                if (_currentLogFileNameCached == null)
                    _currentLogFileNameCached = FileNameBase + _currentLogNumber.ToString() + ".txt";

                return _currentLogFileNameCached;
            }
        }

        protected void IncrementCurrentLogFileNumber()
        {
            _currentLogNumber++;
            _currentLogFileNameCached = null;
        }

        protected void MoveToNextFreeLogFileNumber()
        {
            while (!WantExit && File.Exists(CurrentLogFileName)) { IncrementCurrentLogFileNumber(); }
        }

        protected override void LogListener()
        {
            MoveToNextFreeLogFileNumber();
            List<LogMessage> logEntries = new List<LogMessage>();

            try
            {
                while (!WantExit)
                {
                    while (!WantExit && _logOutput.Count < LogEntriesBeforeWrite) { Thread.Sleep(100); }

                    while (!WantExit && _logOutput.DequeueMultiple(logEntries, LogEntriesBeforeWrite) > 0)
                    {
                        if (_entriesWrittenToCurrentLogFile >= LogEntriesPerFile) { IncrementCurrentLogFileNumber(); }
                        WriteLogEntries(logEntries);
                        logEntries.Clear();
                    }
                }
            }
            catch (ThreadAbortException)
            {
                Thread.CurrentThread.IsBackground = false;
                while (_logOutput.DequeueMultiple(logEntries, LogEntriesBeforeWrite) > 0)
                {
                    WriteLogEntries(logEntries);
                    logEntries.Clear();
                }
                Thread.CurrentThread.IsBackground = true;
            }
            catch (Exception exOther)
            {
                Console.WriteLine(exOther.ToString());
            }
            finally
            {
                Thread.CurrentThread.IsBackground = false;
                while (_logOutput.DequeueMultiple(logEntries, LogEntriesBeforeWrite) > 0)
                {
                    WriteLogEntries(logEntries);
                    logEntries.Clear();
                }
                Thread.CurrentThread.IsBackground = true;
            }
        }

        protected void WriteLogEntries(List<LogMessage> logEntries)
        {
            try
            {
                using (StreamWriter sw = new StreamWriter(CurrentLogFileName, true))
                {
                    logEntries.ForEach(logEntry => sw.WriteLine(logEntry.ToString()));
                    _entriesWrittenToCurrentLogFile += logEntries.Count;
                    sw.Close();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}