﻿// Author - Owen Emlen (owene_1998@yahoo.com)
using System.Runtime.Serialization;
using System;
using System.Xml.Serialization;
using System.ComponentModel;
using System.Threading;
#if USE_HYPER
using Hyper.ComponentModel;
#endif

namespace BrainTechLLC.ThreadSafeObjects
{
#if NO_SILVERLIGHT
    [TypeDescriptionProvider(typeof(HyperTypeDescriptionProvider))]
#endif
    [Serializable]
    [DataContract]
    public class CachedByManualKey<TKey, T> : GenericCacheAbstractBase<T>, ICacheBase<T> where T : class
    {
        public CachedByManualKey() { }
        public CachedByManualKey(TPopulateCacheDelegate<T> populateCacheDelegate) { SetPopulateCacheDelegate(populateCacheDelegate); }

        [NonSerialized, XmlIgnore]
        protected ThreadSafeLookup<TKey, T> _cachedItemsByKey;

        public override bool IsCacheValid() { return (_cachedItemsByKey != null && !_dirty); }
        public override bool IsCacheValid(object populateCacheParameter) { return (!_dirty && _cachedItemsByKey != null && _cachedItemsByKey[(TKey)populateCacheParameter] != null); }

        public virtual T GetItem(TKey key, params object[] args)
        {
            return GetItem((object)key, args);
        }

        /// <summary>
        /// populateCacheParameter is the key for the cached item
        /// </summary>
        /// <param name="populateCacheParameter"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public override T GetItem(object populateCacheParameter, params object[] args)
        {
            T item = null;

            if (IsCacheValid(populateCacheParameter))
            {
                item = _cachedItemsByKey[(TKey)populateCacheParameter];
            }

            if (item == null)
            {
                if (_populateCacheDelegate != null)
                {
                    if (_repopLock == null)
                    {
                        Interlocked.CompareExchange<Lockable>(ref _repopLock, new Lockable(), null);
                    }

                    while (_repopLock.Locked) { Thread.Sleep(10); }

                    _repopLock.AquireLock();
                    {
                        try
                        {
                            if (_dirty)
                            {
                                Interlocked.Exchange<ThreadSafeLookup<TKey, T>>(ref _cachedItemsByKey, null);
                                _dirty = false;
                            }

                            if (!IsCacheValid((TKey)populateCacheParameter))
                            {
                                T outval;
                                if (_populateCacheDelegate(populateCacheParameter, out outval, args))
                                {
                                    if (_cachedItemsByKey == null)
                                    {
                                        Interlocked.Exchange<ThreadSafeLookup<TKey, T>>(ref _cachedItemsByKey, new ThreadSafeLookup<TKey, T>());
                                    }
                                    _cachedItemsByKey.AddOrSet((TKey)populateCacheParameter, outval);
                                    return outval;
                                }
                            }
                        }
                        finally
                        {
                            _repopLock.ReleaseLock();
                        }
                    }
                }
            }
            return item;
        }

        public override T GetItem()
        {
            return GetItem(_defaultPopulateParam, _storedArgs);
        }
    }
}
