﻿using System;
using System.IO;
using System.Diagnostics;
using System.Threading;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BrainTechLLC.FileMaintenance
{
    public enum Response
    {
        None = 0,
        Yes = 1,
        No = 2,
        CapitalNo = 3,
        CapitalYes = 4
    }

    [Flags]
    public enum Options
    {
        None = 0,
        PerformDeletesBasedOnFileSize = 1,
        DoublePass = 2,
        ContinueOnException = 4,
        DeletePSESS = 8,
        RemoveReadOnlyFlags = 16,
        DeleteSVN = 32
    }

    public class FileWorker
    {
        public event EventHandler<ObjectEventArgs<DirectoryInfo>> DeletedDirectory;
        public event EventHandler<ObjectEventArgs<FileInfo>> DeletedFile;

        public List<string> DefaultIgnoreList;
        public bool IgnoreCUsers = true;
        public bool DeleteSVN = false;
        public bool RemoveReadOnlyFlags = true;
        public bool DeletePSESS = false;
        public string SkipDeleteSubdir = string.Empty;
        public string SkipDeleteSubdir2 = string.Empty;

        public FileWorker()
        {
            DefaultIgnoreList = new List<string>()
            {
                "System Volume",               
                "Documents and",                
                "Program Files",                
                "ProgramData",                
                "Windows",                                                
            };

            if (IgnoreCUsers)
                DefaultIgnoreList.Add("c:\\Users");
        }

        public static bool SetFileAttribute(DirectoryInfo source, FileAttributes setAttribute)
        {
            foreach (FileInfo fi in source.GetFiles())
            {
                if (fi.FullName.IndexOf("A525", StringComparison.OrdinalIgnoreCase) >= 0)
                    return false;

                File.SetAttributes(fi.FullName, setAttribute);
            }

            var v = from diSourceSubDir in source.GetDirectories().AsParallel()
                    where !diSourceSubDir.FullName.Equals(source.FullName)
                    select diSourceSubDir;

            v.ForAll(diSourceSubDir => SetFileAttribute(diSourceSubDir, setAttribute));
            //foreach (DirectoryInfo diSourceSubDir in source.GetDirectories())
            //{
            //    if (diSourceSubDir.FullName.Equals(source.FullName)) continue;

            //    SetFileAttribute(diSourceSubDir, setAttribute);
            //}

            return true;
        }

        public void StartNukeBinObjs(string startingDir, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            List<Exception> exceptionsOut = new List<Exception>();
            StartNukeBinObjs(new List<string>() { startingDir }, null, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
        }

        public void StartNukeBinObjs(List<string> startingDirs, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            List<Exception> exceptionsOut = new List<Exception>();
            StartNukeBinObjs(startingDirs, null, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
        }

        public void StartNukeBinObjs(string startingDir, List<string> ignoreList, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            List<Exception> exceptionsOut = new List<Exception>();
            StartNukeBinObjs(new List<string>() { startingDir }, ignoreList, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
        }

        public void StartNukeBinObjs(List<string> startingDirs, List<string> ignoreList, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            List<Exception> exceptionsOut = new List<Exception>();
            StartNukeBinObjs(startingDirs, ignoreList, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
        }

        public void StartNukeBinObjs(string startingDir, List<string> ignoreList, List<Exception> exceptionsOut, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            StartNukeBinObjs(new List<string>() { startingDir }, ignoreList, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
        }

        public void StartNukeBinObjs(List<string> startingDirs, List<string> ignoreList, List<Exception> exceptionsOut, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            if ((options & Options.DoublePass) == Options.DoublePass)
            {
                StartNukeBinObjsInternal(startingDirs, ignoreList, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
                Options options2 = options | Options.PerformDeletesBasedOnFileSize;
                StartNukeBinObjsInternal(startingDirs, ignoreList, exceptionsOut, minSize, maxSize, options2, ConfirmDelete, ConfirmDeleteDirectory, OnException);
            }
            else
            {
                StartNukeBinObjsInternal(startingDirs, ignoreList, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
            }
        }

        private void StartNukeBinObjsInternal(List<string> startingDirs, List<string> ignoreList, List<Exception> exceptionsOut, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            foreach (var dirName in startingDirs)
            {
                try
                {
                    DirectoryInfo di = new DirectoryInfo(dirName);
                    StartNukeBinObjs(di, ignoreList, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
                }
                catch (Exception ex)
                {
                    if ((options & Options.ContinueOnException) != Options.ContinueOnException)
                        throw;

                    exceptionsOut.Add(ex);
                }
            }
        }

        public void StartNukeBinObjs(DirectoryInfo source, List<string> ignoreList, List<Exception> exceptionsOut, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            List<string> allIgnores = new List<string>(DefaultIgnoreList);

            if (ignoreList != null)
                allIgnores.AddRange(ignoreList);

            NukeBinObjs(source, allIgnores, exceptionsOut, minSize, maxSize, options, ConfirmDelete, ConfirmDeleteDirectory, OnException);
        }

        private void NukeBinObjs(DirectoryInfo source, List<string> ignoreList, List<Exception> exceptionsOut, double minSize, double maxSize, Options options, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            NukeBinObjs(source, ignoreList, exceptionsOut, minSize, maxSize, options, Response.None, ConfirmDelete, ConfirmDeleteDirectory, OnException);
        }

        protected void OnDeletedDirectory(DirectoryInfo di)
        {
            if (DeletedDirectory != null)
                DeletedDirectory(this, new ObjectEventArgs<DirectoryInfo>(di));
        }

        protected void OnDeletedFile(FileInfo fi)
        {
            if (DeletedFile != null)
                DeletedFile(this, new ObjectEventArgs<FileInfo>(fi));
        }

        private void NukeBinObjs(DirectoryInfo source, List<string> ignoreList, List<Exception> exceptionsOut, double minSize, double maxSize, Options options, Response prevResponse, Func<FileInfo, Response> ConfirmDelete, Func<DirectoryInfo, Response> ConfirmDeleteDirectory, Action<Exception> OnException)
        {
            if (source.FullName.ContainsAny(ignoreList))
                return;

            bool foundPropertiesDirectory = false;
            bool foundProjectFile = false;

            // Delete .svn directories, if requested
            if (DeleteSVN || (options & Options.DeleteSVN) == Options.DeleteSVN)
            {
                try
                {
                    HandleDeleteSVN(source);
                }
                catch (Exception ex)
                {
                    OnException(ex);
                    exceptionsOut.Add(ex);

                    if ((options & Options.ContinueOnException) != Options.ContinueOnException)
                        throw;
                }
            }

            try
            {
                foundPropertiesDirectory = FindPropertiesDirectory(source);
                foundProjectFile = FindProjectFile(source);
            }
            catch (Exception ex)
            {
                OnException(ex);
                exceptionsOut.Add(ex);

                if ((options & Options.ContinueOnException) != Options.ContinueOnException)
                    throw;
            }

            if ((DeletePSESS || (options & Options.DeletePSESS) == Options.DeletePSESS))
            {
                prevResponse = HandleDeletePSESS(source, exceptionsOut, minSize, maxSize, options, prevResponse, ConfirmDelete, OnException);
            }

            if ((options & Options.PerformDeletesBasedOnFileSize) == Options.PerformDeletesBasedOnFileSize)
            {
                DeleteBasedOnFileSize(source, exceptionsOut, minSize, maxSize, options, ConfirmDelete, OnException);
            }

            try
            {
                foreach (DirectoryInfo diSourceSubDir in source.GetDirectories())
                {
                    if (diSourceSubDir.FullName.Equals(source.FullName)) continue;

                    if ((foundProjectFile || foundPropertiesDirectory) && DirectoryShouldBeDeleted(diSourceSubDir))
                    {
                        if (prevResponse == Response.CapitalNo)
                            continue;

                        if (prevResponse != Response.CapitalYes)
                        {
                            Response r = ConfirmDeleteDirectory(diSourceSubDir);

                            if (!HandleResponse(r, ref prevResponse))
                                continue;
                        }

                        if (WantRemoveReadOnlyFlags(options) && !SetFileAttribute(diSourceSubDir, FileAttributes.Normal))
                            continue;

                        try
                        {
                            Directory.Delete(diSourceSubDir.FullName, true);
                            OnDeletedDirectory(diSourceSubDir);
                        }
                        catch (Exception ex)
                        {
                            OnException(ex);

                            if ((options & Options.ContinueOnException) != Options.ContinueOnException)
                                throw;

                            exceptionsOut.Add(ex);
                        }
                    }
                    else
                    {
                        NukeBinObjs(diSourceSubDir, ignoreList, exceptionsOut, minSize, maxSize, options, prevResponse, ConfirmDelete, ConfirmDeleteDirectory, OnException);
                    }
                }
            }
            catch (Exception ex)
            {
                OnException(ex);
                exceptionsOut.Add(ex);

                if ((options & Options.ContinueOnException) != Options.ContinueOnException)
                    throw;
            }
        }

        private bool WantRemoveReadOnlyFlags(Options options)
        {
            return RemoveReadOnlyFlags || ((options & Options.RemoveReadOnlyFlags) == Options.RemoveReadOnlyFlags);
        }

        private bool DirectoryShouldBeDeleted(DirectoryInfo di)
        {
            bool delete = (di.Name.IndexOf("Backup", StringComparison.OrdinalIgnoreCase) == 0 ||
                           di.Name.Equals("bin", StringComparison.OrdinalIgnoreCase) ||
                           di.Name.Equals("obj", StringComparison.OrdinalIgnoreCase));

            if (delete)
            {
                if (!string.IsNullOrEmpty(SkipDeleteSubdir) && di.FullName.IndexOf(SkipDeleteSubdir, StringComparison.OrdinalIgnoreCase) >= 0)
                    return false;

                if (!string.IsNullOrEmpty(SkipDeleteSubdir2) && di.FullName.IndexOf(SkipDeleteSubdir2, StringComparison.OrdinalIgnoreCase) >= 0)
                    return false;
            }

            return delete;
        }

        private Response HandleDeletePSESS(DirectoryInfo source, List<Exception> exceptionsOut, double minSize, double maxSize, Options options, Response prevResponse, Func<FileInfo, Response> ConfirmDelete, Action<Exception> OnException)
        {
            foreach (FileInfo fi in source.GetFiles())
            {
                if (fi.Extension.IndexOf("psess", StringComparison.OrdinalIgnoreCase) >= 0)
                {
                    if (prevResponse == Response.CapitalNo)
                        continue;

                    if (prevResponse != Response.CapitalYes)
                    {
                        Response r = ConfirmDelete(fi);

                        if (!HandleResponse(r, ref prevResponse))
                            continue;
                    }

                    try
                    {
                        fi.Delete();
                        OnDeletedFile(fi);
                    }
                    catch (Exception e)
                    {
                        OnException(e);
                        exceptionsOut.Add(e);

                        if ((options & Options.ContinueOnException) != Options.ContinueOnException)
                            throw;
                    }
                }
            }
            return prevResponse;
        }

        private static bool FindPropertiesDirectory(DirectoryInfo source)
        {
            try
            {
                foreach (DirectoryInfo diSourceSubDir in source.GetDirectories())
                {
                    // Make sure we find a "Properties" directory, signaling the directory is a visual studio project dir, etc
                    if (diSourceSubDir.Name.Contains("Properties"))
                    {
                        return true;
                    }
                }
            }
            catch
            {
            }

            return false;
        }

        private void HandleDeleteSVN(DirectoryInfo source)
        {
            foreach (DirectoryInfo diSourceSubDir in source.GetDirectories())
            {
                if (diSourceSubDir.Name.Equals(".svn"))
                {
                    SetFileAttribute(diSourceSubDir, FileAttributes.Normal);
                    diSourceSubDir.Delete(true);
                    OnDeletedDirectory(diSourceSubDir);
                    continue;
                }
            }
        }

        private static bool FindProjectFile(DirectoryInfo source)
        {
            try
            {
                foreach (FileInfo fi in source.GetFiles())
                {
                    // Make sure there is a project in the parent directory / again, ensure it's a VS project
                    if (fi.Name.IndexOf(".csproj") > 0 || fi.Name.IndexOf(".vbproj") > 0)
                    {
                        return true;
                    }
                }
            }
            catch
            {
            }

            return false;
        }

        private bool HandleResponse(Response r, ref Response prevResponse)
        {
            if (r == Response.CapitalYes)
            {
                prevResponse = Response.CapitalYes;
            }
            else if (r == Response.CapitalNo)
            {
                prevResponse = Response.CapitalNo;
                return false;
            }
            else if (r != Response.Yes)
            {
                return false;
            }

            return true;
        }

        private static bool DeleteBasedOnFileSize(DirectoryInfo source, List<Exception> exceptionsOut, double min, double max, Options options, Func<FileInfo, Response> ConfirmDelete, Action<Exception> OnException)
        {
            if (min == 0D)
                return true;

            Dictionary<string, bool> askedList = new Dictionary<string, bool>();
            bool stopDir = false;

            for (int size = 0; size < 1; size++)
            {
                if (stopDir)
                    break;

                long minSize = Convert.ToInt64(Math.Floor(1024D * 1024D * min));
                long maxSize = (max == 0D) ? long.MaxValue : Convert.ToInt64(Math.Floor(1024D * 1024D * max));

                foreach (FileInfo fi in source.GetFiles())
                {
                    if (fi.Length >= minSize && fi.Length < maxSize)
                    {
                        if (!askedList.ContainsKey(fi.FullName))
                        {
                            askedList.Add(fi.FullName, true);

                            Response r = ConfirmDelete(fi);

                            if (r == Response.CapitalNo)
                            {
                                stopDir = true;
                                break;
                            }
                            else if (r == Response.No)
                            {
                                continue;
                            }

                            try
                            {
                                File.SetAttributes(fi.FullName, FileAttributes.Normal);
                                fi.Delete();
                            }
                            catch (Exception ex)
                            {
                                OnException(ex);

                                if ((options & Options.ContinueOnException) != Options.ContinueOnException)
                                    throw;

                                exceptionsOut.Add(ex);
                            }
                        }
                    }
                }
            }
            return stopDir;
        }
    }

    public static class Ext
    {
        public static bool ContainsAny(this string originalString, IEnumerable<string> list)
        {
            foreach (var item in list)
            {
                if (originalString.IndexOf(item, StringComparison.OrdinalIgnoreCase) >= 0)
                    return true;
            }

            return false;
        }
    }

    public class ObjectEventArgs<T> : EventArgs where T : class
    {
        public T Item;
        public ObjectEventArgs(T item) { Item = item; }
        public ObjectEventArgs() { }
    }
}