﻿// Original author contact info: Owen Emlen (owene_1998@yahoo.com)
// Note: other individuals may also have contributed to this code
// Project hosted on CodePlex.com as of 1/10/2009 at http://wpfimageviewer.codeplex.com
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using BrainTechLLC;
using System.Windows.Threading;
using System.Windows;
using BrainTechLLC.ThreadSafeObjects;
using System.Threading;
using System.Diagnostics;
using System.Reflection;
using System.Runtime.Serialization;

namespace BrainTechLLC.WPF
{
    [Serializable]
    [DataContract]
    public partial class GenericViewModelBase<TViewModel, T> : ViewModelBase
        where T : BaseNotifyPropertyChanged
        where TViewModel : BaseNotifyPropertyChanged
    {
        public static ThreadSafeListAndLookup<string, string> _alsoRaisePropertiesLookup;

        static GenericViewModelBase()
        {
            _alsoRaisePropertiesLookup = new ThreadSafeListAndLookup<string, string>();
            typeof(T).WireUpAlsoRaisePropertiesLookup(_alsoRaisePropertiesLookup);
            typeof(TViewModel).WireUpAlsoRaisePropertiesLookup(_alsoRaisePropertiesLookup);
        }

        protected T _Item;

        public T Item
        {
            get { return _Item; }
            set
            {
                if (_Item != value)
                {
                    if (_Item != null)
                    {
                        _Item.PropertyChanged -= _Item_PropertyChanged;                        
                    }

                    _Item = value;
                    _Object = value;

                    if (_Item != null)
                    {
                        _Item.PropertyChanged += new PropertyChangedEventHandler(_Item_PropertyChanged);
                    }

                    OnPropertyChangedUIThread("Item");
                }
            }
        }

        protected void _Item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            OnPropertyChangedUIThread(e.PropertyName);
            OnItemRaisedPropertyChanged(e.PropertyName);
        }

        public GenericViewModelBase()
        {
            this.PropertyChanged += new PropertyChangedEventHandler(GenericViewModelBase_PropertyChanged);
        }

        public GenericViewModelBase(T item) : this()
        {
            Item = item;            
        }

        void GenericViewModelBase_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName != null)
            {
                this.CheckAlsoRaise(e.PropertyName, _alsoRaisePropertiesLookup);
            }            
        }

        public override void OnItemRaisedPropertyChanged(string propertyName)
        {
            if (propertyName != null)
            {
                this.CheckAlsoRaise(propertyName, _alsoRaisePropertiesLookup);
            }

            base.OnItemRaisedPropertyChanged(propertyName);
        }
    }
}
