﻿// Original author contact info: Owen Emlen (owene_1998@yahoo.com)
// Note: other individuals may also have contributed to this code
// Project hosted on CodePlex.com as of 1/10/2009 at http://wpfimageviewer.codeplex.com
using System;
using BrainTechLLC.WPF;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;

namespace ImageAppDataModels
{
    public static class TempExten
    {
        public static void HandleChange<T1, T2>(this DependencyObject d, DependencyPropertyChangedEventArgs e, Action<T1, T2> action, Action<T1> update) where T1 : DependencyObject
        {
            T1 ctrl = (T1)d;
            T2 val = (T2)e.NewValue;
            action(ctrl, val);
            update(ctrl);
        }

        public static T FindParent<T>(this FrameworkElement t) where T : FrameworkElement
        {
            if (t == null)
                return null;

            DependencyObject d = VisualTreeHelper.GetParent(t);

            if (d != null)
            {
                T ret = d as T;

                if (ret == null)
                    return (d as FrameworkElement).FindParent<T>();

                return ret;
            }

            return null;
        }

        public static void StretchListBox(this FrameworkElement t)
        {
            ContentPresenter p = t.FindParent<ContentPresenter>();

            if (p != null)
            {
                p.HorizontalAlignment = HorizontalAlignment.Stretch;
            }
        }

        public static bool EnsureVisibility(this UIElement c, bool visible)
        {
            if (c == null)
                return false;

            if (visible && c.Visibility == Visibility.Collapsed)
            {
                c.Visibility = Visibility.Visible;
                return true;
            }
            else if (!visible && c.Visibility == Visibility.Visible)
            {
                c.Visibility = Visibility.Collapsed;
                return true;
            }

            return false;
        }


        //public static List<T> GetAllChildren<T>(this DependencyObject obj, bool recursive) where T : DependencyObject
        //{
        //    List<T> results = new List<T>();

        //    for (int n = 0; n < VisualTreeHelper.GetChildrenCount(obj); n++)
        //    {
        //        var child1 = VisualTreeHelper.GetChild(obj, n);

        //        if (child1 != null)
        //        {
        //            if (child1 is T)
        //                results.Add(child1 as T);

        //            if (recursive)
        //            {
        //                results.AddRange(GetActualChildren<T>(child1 as DependencyObject));
        //            }
        //        }
        //    }

        //    return results;
        //}

        //public static List<T> GetActualChildren<T>(this DependencyObject p) where T : DependencyObject
        //{
        //    List<T> results = new List<T>();

        //    if (p == null)
        //        return results;

        //    if (p is Border)
        //    {
        //        var b = p as Border;
        //        if (b.Child != null)
        //        {
        //            if (b.Child is T)
        //                results.Add(b.Child as T);

        //            results.AddRange(GetActualChildren<T>(b.Child as DependencyObject));
        //        }
        //    }
        //    else if (p is Canvas)
        //    {
        //        results.AddRange((p as Canvas).GetAllChildren<T>(true));
        //    }
        //    else if (p is ItemsPresenter)
        //    {
        //        ItemsPresenter ip = p as ItemsPresenter;
        //        results.AddRange(GetAllChildren<T>(ip, true));
        //    }
        //    else if (p is Panel)
        //    {
        //        results.AddRange((p as Panel).GetAllChildren<T>(true));
        //    }
        //    else if (p is ContentControl)
        //    {
        //        var cc = p as ContentControl;
        //        if (cc.Content != null && cc.Content is DependencyObject)
        //        {
        //            if (cc.Content is T)
        //                results.Add(cc.Content as T);

        //            results.AddRange(GetActualChildren<T>(cc.Content as DependencyObject));
        //        }
        //    }
        //    else if (p is ScrollContentPresenter)
        //    {
        //        var sp = p as ScrollContentPresenter;

        //        if (sp.Content != null && sp.Content is DependencyObject)
        //        {
        //            if (sp.Content is T)
        //                results.Add(sp.Content as T);
        //            results.AddRange(GetActualChildren<T>(sp.Content as DependencyObject));
        //        }
        //    }
        //    else if (p is ContentPresenter && (p as ContentPresenter).Content is DependencyObject)
        //    {
        //        ContentPresenter cp = p as ContentPresenter;
        //        if (cp.Content != null)
        //        {
        //            if (cp.Content is T)
        //                results.Add(cp.Content as T);

        //            results.AddRange(GetActualChildren<T>(cp.Content as DependencyObject));
        //        }
        //    }
        //    else
        //    {
        //        if (p is T)
        //            results.Add(p as T);

        //        results.AddRange(p.GetAllChildren<T>(true));
        //    }

        //    return results;
        //}

        public static int ForEachRecursive<T>(this DependencyObject p, Action<T> action, Func<T, bool> filter) where T : class
        {
            int count = 0;

            if (p == null)
                return 0;

            if (p is Canvas)
                return ForEachRecursive<T>(p as Canvas, action, filter);
            else if (p is Panel)
                return ForEachRecursive<T>(p as Panel, action, filter);

            try
            {
                var results = p.GetActualChildren<DependencyObject>();

                for (int n = 0; n < results.Count; n++)
                {
                    var child1 = results[n];

                    count += ForEachRecursive<T>(child1, action, filter);
                }
            }
            catch { }

            return 0;
        }

        public static int ForEachRecursive<T>(this Canvas p, Action<T> action, Func<T, bool> filter) where T : class
        {
            int count = 0;

            if (p == null)
                return 0;

            if (p is T)
            {
                if (filter == null || !filter(p as T))
                {
                    action(p as T);
                    count++;
                }
            }

            for (int n = 0; n < p.Children.Count; n++)
            {
                var child = p.Children[n];

                if (child is T)
                {
                    if (filter == null || !filter(child as T))
                    {
                        action(child as T);
                        count++;
                    }
                }

                Panel panelChild = child as Panel;
                if (panelChild != null)
                {
                    count += ForEachRecursive<T>(panelChild, action, filter);
                }
                else
                {
                    Canvas can = child as Canvas;

                    if (can != null)
                        count += ForEachRecursive<T>(can, action, filter);
                }

                ContentControl content = child as ContentControl;
                if (content != null && content.Content is Panel)
                {
                    count += ForEachRecursive<T>(content.Content as Panel, action, filter);
                }

                ContentPresenter presenter = child as ContentPresenter;
                if (presenter != null && presenter.Content is Panel)
                {
                    count += ForEachRecursive<T>(presenter.Content as Panel, action, filter);
                }
            }
            return count;
        }

        public static int ForEachRecursive<T>(this Panel p, Action<T> action, Func<T, bool> filter) where T : class
        {
            int count = 0;

            if (p == null)
                return 0;

            if (p is T)
            {
                if (filter == null || !filter(p as T))
                {
                    action(p as T);
                    count++;
                }
            }

            for (int n = 0; n < p.Children.Count; n++)
            {
                var child = p.Children[n];

                if (child is T)
                {
                    if (filter == null || !filter(child as T))
                    {
                        action(child as T);
                        count++;
                    }
                }

                Panel panelChild = child as Panel;
                if (panelChild != null)
                {
                    count += ForEachRecursive<T>(panelChild, action, filter);
                }
                else
                {
                    Canvas can = child as Canvas;

                    if (can != null)
                        count += ForEachRecursive<T>(can, action, filter);
                }

                ContentControl content = child as ContentControl;
                if (content != null && content.Content is Panel)
                {
                    count += ForEachRecursive<T>(content.Content as Panel, action, filter);
                }

                ContentPresenter presenter = child as ContentPresenter;
                if (presenter != null && presenter.Content is Panel)
                {
                    count += ForEachRecursive<T>(presenter.Content as Panel, action, filter);
                }

                ListBox lb = child as ListBox;
                if (lb != null)
                {
                    count += ForEachRecursive<T>(lb as Control, action, filter);
                }
            }
            return count;
        }
    }
}